#*********************************************************
# extract_playlist.py
# Author: Robin Siebler <robinsiebler@321.net>
# Version: 0.1
# Date: 07/19/05
#*********************************************************

"""
extract_playlist.py - extract playlist number and title
from PhatNoise Music Manager .pbx files.
"""

__author = "Robin Siebler"
__date = 07/19/05
__version = 0.1

import os, re, sys
from optparse import OptionParser

re_digits = re.compile(r'(\d+)')

def add_padding(playlist_no, playlist_size):
    """add padding after the number based
    upon the size of the playlist."""
    playlist_no = str(playlist_no) + '.'
    playlist_no = playlist_no.ljust(len(str(playlist_size)) + 2)
    return playlist_no

def embedded_numbers(s):
    """convert digits into numbers so that strings
    will sort properly."""
    # from Python Cookbook 2nd Edition
    # Credit: Sebastien Keim, Chui Tey, Alex Martelli
    pieces = re_digits.split(s)             # split into digits/nondigits
    pieces[1::2] = map(int, pieces[1::2])   # turn digits into numbers
    return pieces

def parse_playlist(file):
    """open a .pbx file and extract the playlist title."""
    opened_file = open(file)
    for line in opened_file:
        if line.find('title') != -1:
            title = line [15:]
            break
    opened_file.close()
    return title
    
def print_err(error):
    print error
    sys.exit(1)
    
def sort_strings(alist):
    return sorted(alist, key=embedded_numbers)

def main():
    err_args = '\nPlease provide a playlist path and output file name.'
    err_out = '\nPlease provide a path and filename for the output file.'
    err_files = '\n\nNo playlist files (.pbx) were found.  \nPlease check the ' +\
                'directory provided and try again.'

    parser = OptionParser()
    parser.add_option('-d', dest='pbx_dir',
                      help='PATH where .pbx files are located ' +\
                      '(PHTDTA\Profiles\Default)', metavar='PATH')
    parser.add_option('-o', dest='output_file',
                      help='FILE to contain playlist', metavar='FILE')
    (options, args) = parser.parse_args()
    if len(sys.argv) < 2:
        parser.print_help()
        print_err(err_args)

    if options.pbx_dir == None:
        pbx_dir = os.getcwd()
    else:
        pbx_dir = options.pbx_dir
        
    if options.output_file == None:
        print_err(err_out)
    else:
        file_name = options.output_file
        
    err_open = '\nAn error occured when trying to open ' + file_name
    err_close = '\nAn error occured when trying to close ' + file_name
    err_write = '\nAn error occured when trying to write to ' + file_name

    os.chdir(pbx_dir)
    pbx_files = []
    pbx_fname = []
    playlist = []
    counter = 0

    print 'Processing playlist files',
    for file in os.listdir(pbx_dir):
        if counter % 100 > 0:
            print '.',
            counter+= 1
        if os.path.isfile(file):
            if os.path.splitext(file)[1] == '.pbx':
                pbx_files.append(os.path.splitext(file)[0])
                pbx_fname.append(file)

    if len(pbx_fname) == 0:
        print_err(err_files)
        
    for file in enumerate(pbx_files):
        if file[0] % 100 > 0:
            print '.',
        playlist_no = ''
        playlist_no = int(file[1].lstrip('p')) + 1 # get playlist number
        playlist_no = add_padding(playlist_no, len(pbx_files))
        title = parse_playlist(pbx_fname[file[0]]) # get playlist title
        playlist.append(str(playlist_no) + title)
    
    playlist = sort_strings(playlist)
    
    try:
        playlist_file = open(file_name, 'w')
    except:
        print_err(err_open)
        
    try:
        playlist_file.writelines(playlist)
    except:
        print_err(err_write)

    try:
        playlist_file.close()
    except:
        print_err(err_close)
    
    print '\n\nPlaylist created successfully!\n'

if __name__ == '__main__':
    main()